@@ -8,7 +8,7 @@ __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
 # Imports {{{
-import os, shutil, uuid, json, glob, time, cPickle, hashlib, errno, sys
+import os, shutil, uuid, json, glob, time, hashlib, errno, sys
 from functools import partial
 
 import apsw
@@ -23,6 +23,7 @@ from calibre.db.delete_service import delete_service
 from calibre.db.errors import NoSuchFormat
 from calibre.library.field_metadata import FieldMetadata
 from calibre.ebooks.metadata import title_sort, author_to_author_sort
+from calibre.utils import pickle_binary_string, unpickle_binary_string
 from calibre.utils.icu import sort_key
 from calibre.utils.config import to_json, from_json, prefs, tweaks
 from calibre.utils.date import utcfromtimestamp, parse_date
@@ -30,8 +31,11 @@ from calibre.utils.filenames import (
     is_case_sensitive, samefile, hardlink_file, ascii_filename,
     WindowsAtomicFolderMove, atomic_rename, remove_dir_if_empty,
     copytree_using_links, copyfile_using_links)
-from calibre.utils.magick.draw import save_cover_data_to
-from calibre.utils.formatter_functions import load_user_template_functions
+from calibre.utils.img import save_cover_data_to
+from calibre.utils.formatter_functions import (load_user_template_functions,
+            unload_user_template_functions,
+            compile_user_template_functions,
+            formatter_functions)
 from calibre.db.tables import (OneToOneTable, ManyToOneTable, ManyToManyTable,
         SizeTable, FormatsTable, AuthorsTable, IdentifiersTable, PathTable,
         CompositeTable, UUIDTable, RatingTable)
@@ -45,8 +49,8 @@ Differences in semantics from pysqlite:
     3. There is no executescript
 
 '''
-CUSTOM_DATA_TYPES = frozenset(['rating', 'text', 'comments', 'datetime',
-    'int', 'float', 'bool', 'series', 'composite', 'enumeration'])
+CUSTOM_DATA_TYPES = frozenset(('rating', 'text', 'comments', 'datetime',
+    'int', 'float', 'bool', 'series', 'composite', 'enumeration'))
 WINDOWS_RESERVED_NAMES = frozenset('CON PRN AUX NUL COM1 COM2 COM3 COM4 COM5 COM6 COM7 COM8 COM9 LPT1 LPT2 LPT3 LPT4 LPT5 LPT6 LPT7 LPT8 LPT9'.split())
 
 
@@ -65,6 +69,7 @@ class DynamicFilter(object):  # {{{
         self.ids = frozenset(ids)
 # }}}
 
+
 class DBPrefs(dict):  # {{{
 
     'Store preferences as key:value pairs in the db'
@@ -160,6 +165,8 @@ class DBPrefs(dict):  # {{{
 # }}}
 
 # Extra collators {{{
+
+
 def pynocase(one, two, encoding='utf-8'):
     if isbytestring(one):
         try:
@@ -173,11 +180,13 @@ def pynocase(one, two, encoding='utf-8'):
             pass
     return cmp(one.lower(), two.lower())
 
+
 def _author_to_author_sort(x):
     if not x:
         return ''
     return author_to_author_sort(x.replace('|', ','))
 
+
 def icu_collator(s1, s2):
     return cmp(sort_key(force_unicode(s1, 'utf-8')),
                sort_key(force_unicode(s2, 'utf-8')))
@@ -185,6 +194,8 @@ def icu_collator(s1, s2):
 # }}}
 
 # Unused aggregators {{{
+
+
 def Concatenate(sep=','):
     '''String concatenation aggregator for sqlite'''
 
@@ -199,6 +210,7 @@ def Concatenate(sep=','):
 
     return ([], step, finalize)
 
+
 def SortedConcatenate(sep=','):
     '''String concatenation aggregator for sqlite, sorted by supplied index'''
 
@@ -213,6 +225,7 @@ def SortedConcatenate(sep=','):
 
     return ({}, step, finalize)
 
+
 def IdentifiersConcat():
     '''String concatenation aggregator for the identifiers map'''
 
@@ -224,6 +237,7 @@ def IdentifiersConcat():
 
     return ([], step, finalize)
 
+
 def AumSortedConcatenate():
     '''String concatenation aggregator for the author sort map'''
 
@@ -244,6 +258,7 @@ def AumSortedConcatenate():
 
 # }}}
 
+
 class Connection(apsw.Connection):  # {{{
 
     BUSY_TIMEOUT = 10000  # milliseconds
@@ -252,7 +267,7 @@ class Connection(apsw.Connection):  # {{{
         apsw.Connection.__init__(self, path)
 
         self.setbusytimeout(self.BUSY_TIMEOUT)
-        self.execute('pragma cache_size=5000')
+        self.execute('pragma cache_size=-5000')
         self.execute('pragma temp_store=2')
 
         encoding = self.execute('pragma encoding').next()[0]
@@ -304,6 +319,12 @@ class Connection(apsw.Connection):  # {{{
 
 # }}}
 
+
+def set_global_state(backend):
+    load_user_template_functions(
+        backend.library_id, (), precompiled_user_functions=backend.get_user_template_functions())
+
+
 class DB(object):
 
     PATH_LIMIT = 40 if iswindows else 100
@@ -336,8 +357,7 @@ class DB(object):
         if not exists:
             # Be more strict when creating new libraries as the old calculation
             # allowed for max path lengths of 265 chars.
-            if (iswindows and len(self.library_path) >
-                    self.WINDOWS_LIBRARY_PATH_LIMIT):
+            if (iswindows and len(self.library_path) > self.WINDOWS_LIBRARY_PATH_LIMIT):
                 raise ValueError(_(
                     'Path to library too long. Must be less than'
                     ' %d characters.')%self.WINDOWS_LIBRARY_PATH_LIMIT)
@@ -389,9 +409,21 @@ class DB(object):
         self.initialize_prefs(default_prefs, restore_all_prefs, progress_callback)
         self.initialize_custom_columns()
         self.initialize_tables()
+        self.set_user_template_functions(compile_user_template_functions(
+                                 self.prefs.get('user_template_functions', [])))
         if load_user_formatter_functions:
-            load_user_template_functions(self.library_id,
-                                        self.prefs.get('user_template_functions', []))
+            set_global_state(self)
+
+    def get_template_functions(self):
+        return self._template_functions
+
+    def get_user_template_functions(self):
+        return self._user_template_functions
+
+    def set_user_template_functions(self, user_formatter_functions):
+        self._user_template_functions = user_formatter_functions
+        self._template_functions = formatter_functions().get_builtins().copy()
+        self._template_functions.update(user_formatter_functions)
 
     def initialize_prefs(self, default_prefs, restore_all_prefs, progress_callback):  # {{{
         self.prefs = DBPrefs(self)
@@ -412,8 +444,7 @@ class DB(object):
                     progress_callback(_('creating custom column ') + f['label'], i)
                     self.create_custom_column(f['label'], f['name'],
                             f['datatype'],
-                            (f['is_multiple'] is not None and
-                                len(f['is_multiple']) > 0),
+                            (f['is_multiple'] is not None and len(f['is_multiple']) > 0),
                             f['is_editable'], f['display'])
 
         defs = self.prefs.defaults
@@ -432,19 +463,23 @@ class DB(object):
         defs['similar_series_search_key'] = 'series'
         defs['similar_series_match_kind'] = 'match_any'
         defs['book_display_fields'] = [
-        ('title', False), ('authors', True), ('formats', True),
-        ('series', True), ('identifiers', True), ('tags', True),
+        ('title', False), ('authors', True), ('series', True),
+        ('identifiers', True), ('tags', True), ('formats', True),
         ('path', True), ('publisher', False), ('rating', False),
         ('author_sort', False), ('sort', False), ('timestamp', False),
         ('uuid', False), ('comments', True), ('id', False), ('pubdate', False),
         ('last_modified', False), ('size', False), ('languages', False),
         ]
+        defs['popup_book_display_fields'] = [('title', True)] + [(f[0], True) for f in defs['book_display_fields'] if f[0] != 'title']
+        defs['qv_display_fields'] = [('title', True), ('authors', True), ('series', True)]
         defs['virtual_libraries'] = {}
         defs['virtual_lib_on_startup'] = defs['cs_virtual_lib_on_startup'] = ''
         defs['virt_libs_hidden'] = defs['virt_libs_order'] = ()
         defs['update_all_last_mod_dates_on_start'] = False
         defs['field_under_covers_in_grid'] = 'title'
         defs['cover_browser_title_template'] = '{title}'
+        defs['cover_browser_subtitle_field'] = 'rating'
+        defs['styled_columns'] = {}
 
         # Migrate the bool tristate tweak
         defs['bools_are_tristate'] = \
@@ -872,7 +907,7 @@ class DB(object):
         import re
         if not label:
             raise ValueError(_('No label was provided'))
-        if re.match('^\w*$', label) is None or not label[0].isalpha() or label.lower() != label:
+        if re.match(r'^\w*$', label) is None or not label[0].isalpha() or label.lower() != label:
             raise ValueError(_('The label must contain only lower case letters, digits and underscores, and start with a letter'))
         if datatype not in CUSTOM_DATA_TYPES:
             raise ValueError('%r is not a supported data type'%datatype)
@@ -1030,13 +1065,18 @@ class DB(object):
         data = self.custom_field_metadata(label, num)
         self.execute('UPDATE custom_columns SET mark_for_delete=1 WHERE id=?', (data['num'],))
 
-    def close(self, force=False):
+    def close(self, force=False, unload_formatter_functions=True):
         if getattr(self, '_conn', None) is not None:
+            if unload_formatter_functions:
+                try:
+                    unload_user_template_functions(self.library_id)
+                except Exception:
+                    pass
             self._conn.close(force)
             del self._conn
 
     def reopen(self, force=False):
-        self.close(force)
+        self.close(force=force, unload_formatter_functions=False)
         self._conn = None
         self.conn
 
@@ -1065,7 +1105,7 @@ class DB(object):
                     shell.process_command('.read ' + fname.replace(os.sep, '/'))
                     conn.execute('PRAGMA user_version=%d;'%uv)
 
-                self.close()
+                self.close(unload_formatter_functions=False)
                 try:
                     atomic_rename(tmpdb, self.dbpath)
                 finally:
@@ -1376,7 +1416,7 @@ class DB(object):
                     os.remove(path)
         else:
             if no_processing:
-                with open(path, 'wb') as f:
+                with lopen(path, 'wb') as f:
                     f.write(data)
             else:
                 try:
@@ -1620,6 +1660,7 @@ class DB(object):
 
     def get_custom_book_data(self, name, book_ids, default=None):
         book_ids = frozenset(book_ids)
+
         def safe_load(val):
             try:
                 return json.loads(val, object_hook=from_json)
@@ -1653,7 +1694,10 @@ class DB(object):
     def conversion_options(self, book_id, fmt):
         for (data,) in self.conn.get('SELECT data FROM conversion_options WHERE book=? AND format=?', (book_id, fmt.upper())):
             if data:
-                return cPickle.loads(bytes(data))
+                try:
+                    return unpickle_binary_string(bytes(data))
+                except Exception:
+                    pass
 
     def has_conversion_options(self, ids, fmt='PIPE'):
         ids = frozenset(ids)
@@ -1670,7 +1714,8 @@ class DB(object):
             [(book_id, fmt.upper()) for book_id in book_ids])
 
     def set_conversion_options(self, options, fmt):
-        options = [(book_id, fmt.upper(), buffer(cPickle.dumps(data, -1))) for book_id, data in options.iteritems()]
+        options = [(book_id, fmt.upper(), buffer(pickle_binary_string(data.encode('utf-8') if isinstance(data, unicode) else data)))
+                for book_id, data in options.iteritems()]
         self.executemany('INSERT OR REPLACE INTO conversion_options(book,format,data) VALUES (?,?,?)', options)
 
     def get_top_level_move_items(self, all_paths):
@@ -1744,14 +1789,13 @@ class DB(object):
         self.executemany('INSERT INTO data (book,format,uncompressed_size,name) VALUES (?,?,?,?)', vals)
 
     def backup_database(self, path):
-        # We have to open a new connection to self.dbpath, until this issue is fixed:
-        # https://github.com/rogerbinns/apsw/issues/199
         dest_db = apsw.Connection(path)
-        source = apsw.Connection(self.dbpath)
-        with dest_db.backup('main', source, 'main') as b:
+        with dest_db.backup('main', self.conn, 'main') as b:
             while not b.done:
-                b.step(100)
-        source.close()
+                try:
+                    b.step(100)
+                except apsw.BusyError:
+                    pass
         dest_db.cursor().execute('DELETE FROM metadata_dirtied; VACUUM;')
         dest_db.close()
 
