@@ -24,9 +24,11 @@ if ispy3:
 
 # Convert data into values suitable for the db {{{
 
+
 def sqlite_datetime(x):
     return isoformat(x, sep=' ') if isinstance(x, datetime) else x
 
+
 def single_text(x):
     if x is None:
         return x
@@ -35,8 +37,10 @@ def single_text(x):
     x = x.strip()
     return x if x else None
 
+
 series_index_pat = re.compile(r'(.*)\s+\[([.0-9]+)\]$')
 
+
 def get_series_values(val):
     if not val:
         return (val, None)
@@ -50,6 +54,7 @@ def get_series_values(val):
             pass
     return (val, None)
 
+
 def multiple_text(sep, ui_sep, x):
     if not x:
         return ()
@@ -65,6 +70,7 @@ def multiple_text(sep, ui_sep, x):
     x = (y.strip().replace(ui_sep, repsep) for y in x if y.strip())
     return tuple(' '.join(y.split()) for y in x if y)
 
+
 def adapt_datetime(x):
     if isinstance(x, (unicode, bytes)):
         x = parse_date(x, assume_utc=False, as_utc=False)
@@ -72,6 +78,7 @@ def adapt_datetime(x):
         x = UNDEFINED_DATE
     return x
 
+
 def adapt_date(x):
     if isinstance(x, (unicode, bytes)):
         x = parse_only_date(x)
@@ -79,6 +86,7 @@ def adapt_date(x):
         x = UNDEFINED_DATE
     return x
 
+
 def adapt_number(typ, x):
     if x is None:
         return None
@@ -87,6 +95,7 @@ def adapt_number(typ, x):
             return None
     return typ(x)
 
+
 def adapt_bool(x):
     if isinstance(x, (unicode, bytes)):
         x = x.lower()
@@ -100,6 +109,7 @@ def adapt_bool(x):
             x = bool(int(x))
     return x if x is None else bool(x)
 
+
 def adapt_languages(to_tuple, x):
     ans = []
     for lang in to_tuple(x):
@@ -109,11 +119,13 @@ def adapt_languages(to_tuple, x):
         ans.append(lc)
     return tuple(ans)
 
+
 def clean_identifier(typ, val):
     typ = icu_lower(typ or '').strip().replace(':', '').replace(',', '')
     val = (val or '').strip().replace(',', '|')
     return typ, val
 
+
 def adapt_identifiers(to_tuple, x):
     if not isinstance(x, dict):
         x = {k:v for k, v in (y.partition(':')[0::2] for y in to_tuple(x))}
@@ -124,10 +136,12 @@ def adapt_identifiers(to_tuple, x):
             ans[k] = v
     return ans
 
+
 def adapt_series_index(x):
     ret = adapt_number(float, x)
     return 1.0 if ret is None else ret
 
+
 def get_adapter(name, metadata):
     dt = metadata['datatype']
     if dt == 'text':
@@ -174,6 +188,8 @@ def get_adapter(name, metadata):
 # }}}
 
 # One-One fields {{{
+
+
 def one_one_in_books(book_id_val_map, db, field, *args):
     'Set a one-one field in the books table'
     if book_id_val_map:
@@ -183,10 +199,12 @@ def one_one_in_books(book_id_val_map, db, field, *args):
         field.table.book_col_map.update(book_id_val_map)
     return set(book_id_val_map)
 
+
 def set_uuid(book_id_val_map, db, field, *args):
     field.table.update_uuid_cache(book_id_val_map)
     return one_one_in_books(book_id_val_map, db, field, *args)
 
+
 def set_title(book_id_val_map, db, field, *args):
     ans = one_one_in_books(book_id_val_map, db, field, *args)
     # Set the title sort field
@@ -194,6 +212,7 @@ def set_title(book_id_val_map, db, field, *args):
         {k:title_sort(v) for k, v in book_id_val_map.iteritems()}, db)
     return ans
 
+
 def one_one_in_other(book_id_val_map, db, field, *args):
     'Set a one-one field in the non-books table, like comments'
     deleted = tuple((k,) for k, v in book_id_val_map.iteritems() if v is None)
@@ -210,6 +229,7 @@ def one_one_in_other(book_id_val_map, db, field, *args):
         field.table.book_col_map.update(updated)
     return set(book_id_val_map)
 
+
 def custom_series_index(book_id_val_map, db, field, *args):
     series_field = field.series_field
     sequence = []
@@ -228,12 +248,14 @@ def custom_series_index(book_id_val_map, db, field, *args):
 
 # Many-One fields {{{
 
+
 def safe_lower(x):
     try:
         return icu_lower(x)
     except (TypeError, ValueError, KeyError, AttributeError):
         return x
 
+
 def get_db_id(val, db, m, table, kmap, rid_map, allow_case_change,
               case_changes, val_map, is_authors=False):
     ''' Get the db id for the value val. If val does not exist in the db it is
@@ -258,6 +280,7 @@ def get_db_id(val, db, m, table, kmap, rid_map, allow_case_change,
         case_changes[item_id] = val
     val_map[val] = item_id
 
+
 def change_case(case_changes, dirtied, db, table, m, is_authors=False):
     if is_authors:
         vals = ((val.replace(',', '|'), item_id) for item_id, val in
@@ -272,6 +295,7 @@ def change_case(case_changes, dirtied, db, table, m, is_authors=False):
         if is_authors:
             table.asort_map[item_id] = author_to_author_sort(val)
 
+
 def many_one(book_id_val_map, db, field, allow_case_change, *args):
     dirtied = set()
     m = field.metadata
@@ -347,6 +371,7 @@ def many_one(book_id_val_map, db, field, allow_case_change, *args):
 
 # Many-Many fields {{{
 
+
 def uniq(vals, kmap=lambda x:x):
     ''' Remove all duplicates from vals, while preserving order. kmap must be a
     callable that returns a hashable value for every item in vals '''
@@ -356,6 +381,7 @@ def uniq(vals, kmap=lambda x:x):
     seen_add = seen.add
     return tuple(x for x, k in zip(vals, lvals) if k not in seen and not seen_add(k))
 
+
 def many_many(book_id_val_map, db, field, allow_case_change, *args):
     dirtied = set()
     m = field.metadata
@@ -450,6 +476,7 @@ def many_many(book_id_val_map, db, field, allow_case_change, *args):
 
 # }}}
 
+
 def identifiers(book_id_val_map, db, field, *args):  # {{{
     table = field.table
     updates = set()
@@ -475,9 +502,11 @@ def identifiers(book_id_val_map, db, field, *args):  # {{{
     return set(book_id_val_map)
 # }}}
 
+
 def dummy(book_id_val_map, *args):
     return set()
 
+
 class Writer(object):
 
     def __init__(self, field):
@@ -500,8 +529,7 @@ class Writer(object):
         elif field.is_many_many:
             self.set_books_func = many_many
         elif field.is_many:
-            self.set_books_func = (self.set_books_for_enum if dt ==
-                                   'enumeration' else many_one)
+            self.set_books_func = (self.set_books_for_enum if dt == 'enumeration' else many_one)
         else:
             self.set_books_func = (one_one_in_books if field.metadata['table'] == 'books' else one_one_in_other)
             if self.name in {'timestamp', 'uuid', 'sort'}:
@@ -524,5 +552,3 @@ class Writer(object):
         if not book_id_val_map:
             return set()
         return many_one(book_id_val_map, db, field, False)
-
-
