@@ -10,27 +10,33 @@ from various formats.
 import traceback, os, re
 from calibre import CurrentDir, prints
 
+
 class ConversionError(Exception):
 
     def __init__(self, msg, only_msg=False):
         Exception.__init__(self, msg)
         self.only_msg = only_msg
 
+
 class UnknownFormatError(Exception):
     pass
 
+
 class DRMError(ValueError):
     pass
 
+
 class ParserError(ValueError):
     pass
 
+
 BOOK_EXTENSIONS = ['lrf', 'rar', 'zip', 'rtf', 'lit', 'txt', 'txtz', 'text', 'htm', 'xhtm',
                    'html', 'htmlz', 'xhtml', 'pdf', 'pdb', 'updb', 'pdr', 'prc', 'mobi', 'azw', 'doc',
-                   'epub', 'fb2', 'djv', 'djvu', 'lrx', 'cbr', 'cbz', 'cbc', 'oebzip',
+                   'epub', 'fb2', 'fbz', 'djv', 'djvu', 'lrx', 'cbr', 'cbz', 'cbc', 'oebzip',
                    'rb', 'imp', 'odt', 'chm', 'tpz', 'azw1', 'pml', 'pmlz', 'mbp', 'tan', 'snb',
                    'xps', 'oxps', 'azw4', 'book', 'zbf', 'pobi', 'docx', 'docm', 'md',
-                   'textile', 'markdown', 'ibook', 'ibooks', 'iba', 'azw3', 'ps', 'kepub']
+                   'textile', 'markdown', 'ibook', 'ibooks', 'iba', 'azw3', 'ps', 'kepub', 'kfx']
+
 
 class HTMLRenderer(object):
 
@@ -74,6 +80,7 @@ def return_raster_image(path):
         if what(None, raw) not in (None, 'svg'):
             return raw
 
+
 def extract_cover_from_embedded_svg(html, base, log):
     from lxml import etree
     from calibre.ebooks.oeb.base import XPath, SVG, XLINK
@@ -87,6 +94,7 @@ def extract_cover_from_embedded_svg(html, base, log):
             path = os.path.join(base, *href.split('/'))
             return return_raster_image(path)
 
+
 def extract_calibre_cover(raw, base, log):
     from calibre.ebooks.BeautifulSoup import BeautifulSoup
     soup = BeautifulSoup(raw)
@@ -114,6 +122,7 @@ def extract_calibre_cover(raw, base, log):
                 img = os.path.join(base, *images[0]['src'].split('/'))
                 return return_raster_image(img)
 
+
 def render_html_svg_workaround(path_to_html, log, width=590, height=750):
     from calibre.ebooks.oeb.base import SVG_NS
     raw = open(path_to_html, 'rb').read()
@@ -148,21 +157,23 @@ def render_html_svg_workaround(path_to_html, log, width=590, height=750):
                 traceback.print_exc()
     return data
 
+
 def render_html_data(path_to_html, width, height):
     renderer = render_html(path_to_html, width, height)
     return getattr(renderer, 'data', None)
 
+
 def render_html(path_to_html, width=590, height=750, as_xhtml=True):
     from PyQt5.QtWebKitWidgets import QWebPage
     from PyQt5.Qt import QEventLoop, QPalette, Qt, QUrl, QSize
-    from calibre.gui2 import is_ok_to_use_qt
+    from calibre.gui2 import is_ok_to_use_qt, secure_web_page
     if not is_ok_to_use_qt():
         return None
     path_to_html = os.path.abspath(path_to_html)
     with CurrentDir(os.path.dirname(path_to_html)):
         page = QWebPage()
         settings = page.settings()
-        settings.setAttribute(settings.PluginsEnabled, False)
+        secure_web_page(settings)
         pal = page.palette()
         pal.setBrush(QPalette.Background, Qt.white)
         page.setPalette(pal)
@@ -187,6 +198,7 @@ def render_html(path_to_html, width=590, height=750, as_xhtml=True):
                 as_xhtml=False)
     return renderer
 
+
 def check_ebook_format(stream, current_guess):
     ans = current_guess
     if current_guess.lower() in ('prc', 'mobi', 'azw', 'azw1', 'azw3'):
@@ -196,12 +208,14 @@ def check_ebook_format(stream, current_guess):
         stream.seek(0)
     return ans
 
+
 def normalize(x):
     if isinstance(x, unicode):
         import unicodedata
         x = unicodedata.normalize('NFC', x)
     return x
 
+
 def calibre_cover(title, author_string, series_string=None,
         output_format='jpg', title_size=46, author_size=36, logo_path=None):
     title = normalize(title)
@@ -212,8 +226,10 @@ def calibre_cover(title, author_string, series_string=None,
     ans = calibre_cover2(title, author_string or '', series_string or '', logo_path=logo_path, as_qimage=True)
     return image_to_data(ans, fmt=output_format)
 
+
 UNIT_RE = re.compile(r'^(-*[0-9]*[.]?[0-9]*)\s*(%|em|ex|en|px|mm|cm|in|pt|pc|rem|q)$')
 
+
 def unit_convert(value, base, font, dpi, body_font_size=12):
     ' Return value in pts'
     if isinstance(value, (int, long, float)):
@@ -254,6 +270,7 @@ def unit_convert(value, base, font, dpi, body_font_size=12):
             result = value * 0.708661417325
     return result
 
+
 def parse_css_length(value):
     try:
         m = UNIT_RE.match(value)
@@ -265,6 +282,7 @@ def parse_css_length(value):
         return value, unit.lower()
     return None, None
 
+
 def generate_masthead(title, output_path=None, width=600, height=60):
     from calibre.ebooks.conversion.config import load_defaults
     recs = load_defaults('mobi_output')
@@ -272,6 +290,7 @@ def generate_masthead(title, output_path=None, width=600, height=60):
     from calibre.ebooks.covers import generate_masthead
     return generate_masthead(title, output_path=output_path, width=width, height=height, font_family=masthead_font_family)
 
+
 def escape_xpath_attr(value):
     if '"' in value:
         if "'" in value:
