@@ -9,6 +9,7 @@ __docformat__ = 'restructuredtext en'
 from calibre.customize.conversion import (OutputFormatPlugin,
         OptionRecommendation)
 
+
 def remove_html_cover(oeb, log):
     from calibre.ebooks.oeb.base import OEB_DOCS
 
@@ -27,17 +28,21 @@ def remove_html_cover(oeb, log):
         if item.media_type in OEB_DOCS:
             oeb.manifest.remove(item)
 
+
 def extract_mobi(output_path, opts):
     if opts.extract_to is not None:
         from calibre.ebooks.mobi.debug.main import inspect_mobi
         ddir = opts.extract_to
         inspect_mobi(output_path, ddir=ddir)
 
+
 class MOBIOutput(OutputFormatPlugin):
 
     name = 'MOBI Output'
     author = 'Kovid Goyal'
     file_type = 'mobi'
+    commit_name = 'mobi_output'
+    ui_data = {'file_types': ['old', 'both', 'new']}
 
     options = set([
         OptionRecommendation(name='prefer_author_sort',
@@ -92,14 +97,14 @@ class MOBIOutput(OutputFormatPlugin):
                 'formats. This option tells calibre not to do this. '
                 'Useful if your document contains lots of GIF/PNG images that '
                 'become very large when converted to JPEG.')),
-        OptionRecommendation(name='mobi_file_type', choices=['old', 'both',
-            'new'], recommended_value='old',
+        OptionRecommendation(name='mobi_file_type', choices=ui_data['file_types'], recommended_value='old',
             help=_('By default calibre generates MOBI files that contain the '
                 'old MOBI 6 format. This format is compatible with all '
                 'devices. However, by changing this setting, you can tell '
                 'calibre to generate MOBI files that contain both MOBI 6 and '
                 'the new KF8 format, or only the new KF8 format. KF8 has '
-                'more features than MOBI 6, but only works with newer Kindles.')),
+                'more features than MOBI 6, but only works with newer Kindles. '
+                'Allowed values: {}').format('old, both, new')),
 
     ])
 
@@ -261,11 +266,13 @@ class MOBIOutput(OutputFormatPlugin):
                 for td in cols:
                     td.tag = XHTML('span' if cols else 'div')
 
+
 class AZW3Output(OutputFormatPlugin):
 
     name = 'AZW3 Output'
     author = 'Kovid Goyal'
     file_type = 'azw3'
+    commit_name = 'azw3_output'
 
     options = set([
         OptionRecommendation(name='prefer_author_sort',
@@ -327,5 +334,3 @@ class AZW3Output(OutputFormatPlugin):
     def specialize_css_for_output(self, log, opts, item, stylizer):
         from calibre.ebooks.mobi.writer8.cleanup import CSSCleanup
         CSSCleanup(log, opts)(item, stylizer)
-
-
