@@ -15,6 +15,7 @@ from collections import OrderedDict, defaultdict
 from calibre.ebooks.mobi.utils import (encint, encode_number_as_hex,
         encode_tbs, align_block, RECORD_SIZE, CNCX as CNCX_)
 
+
 class CNCX(CNCX_):  # {{{
 
     def __init__(self, toc, is_periodical):
@@ -30,6 +31,7 @@ class CNCX(CNCX_):  # {{{
         CNCX_.__init__(self, strings)
 # }}}
 
+
 class TAGX(object):  # {{{
 
     BITMASKS = {11:0b1}
@@ -132,6 +134,7 @@ class IndexEntry(object):
     def size(self):
         def fget(self):
             return self.length
+
         def fset(self, val):
             self.length = val
         return property(fget=fget, fset=fset, doc='Alias for length')
@@ -198,6 +201,7 @@ class IndexEntry(object):
         ans = buf.getvalue()
         return ans
 
+
 class PeriodicalIndexEntry(IndexEntry):
 
     def __init__(self, offset, label_offset, class_offset, depth):
@@ -206,6 +210,7 @@ class PeriodicalIndexEntry(IndexEntry):
         self.class_offset = class_offset
         self.control_byte_count = 2
 
+
 class SecondaryIndexEntry(IndexEntry):
 
     INDEX_MAP = {'author':73, 'caption':72, 'credit':71, 'description':70,
@@ -238,6 +243,7 @@ class SecondaryIndexEntry(IndexEntry):
 
 # }}}
 
+
 class TBS(object):  # {{{
 
     '''
@@ -308,8 +314,7 @@ class TBS(object):  # {{{
 
             # parent_section_index is needed for the last record
             if first_node is not None and first_node.depth > 0:
-                parent_section_index = (first_node.index if first_node.depth
-                        == 1 else first_node.parent_index)
+                parent_section_index = (first_node.index if first_node.depth == 1 else first_node.parent_index)
             else:
                 parent_section_index = max(self.section_map.iterkeys())
 
@@ -340,8 +345,7 @@ class TBS(object):  # {{{
             extra = {}
             # Write starting section information
             if spanner is None:
-                num_articles = len([a for a in depth_map[1] if a.parent_index
-                    == parent_section_index])
+                num_articles = len([a for a in depth_map[1] if a.parent_index == parent_section_index])
                 if not depth_map[1]:
                     extra = {0b0001: 0}
                 if num_articles > 1:
@@ -424,6 +428,7 @@ class TBS(object):  # {{{
 
 # }}}
 
+
 class Indexer(object):  # {{{
 
     def __init__(self, serializer, number_of_text_records,
