@@ -54,7 +54,11 @@ class ChooseFormatDialog(QDialog):
         menu = self.own
         menu.clear()
         fmt = self._formats[self.formats.currentRow()]
-        populate_menu(menu, self.open_with, fmt)
+
+        def connect_action(ac, entry):
+            connect_lambda(ac.triggered, self, lambda self: self.open_with(entry))
+
+        populate_menu(menu, connect_action, fmt)
         if len(menu.actions()) == 0:
             menu.addAction(_('Open %s with...') % fmt.upper(), self.choose_open_with)
         else:
