@@ -9,7 +9,6 @@ import json, textwrap
 
 from collections import defaultdict
 from threading import Thread
-from functools import partial
 
 from PyQt5.Qt import (
     QApplication, QFont, QFontInfo, QFontDialog, QColorDialog, QPainter,
@@ -111,7 +110,7 @@ class IdLinksRuleEdit(Dialog):
     def __init__(self, key='', name='', template='', parent=None):
         title = _('Edit rule') if key else _('Create a new rule')
         Dialog.__init__(self, title=title, name='id-links-rule-editor', parent=parent)
-        self.key.setText(key), self.nw.setText(name), self.template.setText(template or 'http://example.com/{id}')
+        self.key.setText(key), self.nw.setText(name), self.template.setText(template or 'https://example.com/{id}')
 
     @property
     def rule(self):
@@ -172,13 +171,13 @@ class IdLinksEditor(Dialog):
         l.addWidget(t)
         t.horizontalHeader().setSectionResizeMode(2, t.horizontalHeader().Stretch)
         self.cb = b = QPushButton(QIcon(I('plus.png')), _('&Add rule'), self)
-        b.clicked.connect(lambda : self.edit_rule())
+        connect_lambda(b.clicked, self, lambda self: self.edit_rule())
         self.bb.addButton(b, self.bb.ActionRole)
         self.rb = b = QPushButton(QIcon(I('minus.png')), _('&Remove rule'), self)
-        b.clicked.connect(lambda : self.remove_rule())
+        connect_lambda(b.clicked, self, lambda self: self.remove_rule())
         self.bb.addButton(b, self.bb.ActionRole)
         self.eb = b = QPushButton(QIcon(I('modified.png')), _('&Edit rule'), self)
-        b.clicked.connect(lambda : self.edit_rule(self.table.currentRow()))
+        connect_lambda(b.clicked, self, lambda self: self.edit_rule(self.table.currentRow()))
         self.bb.addButton(b, self.bb.ActionRole)
         l.addWidget(self.bb)
 
@@ -394,6 +393,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
             self.opt_hidpi.setVisible(False), self.label_hidpi.setVisible(False)
         r('ui_style', gprefs, restart_required=True, choices=[(_('System default'), 'system'), (_('calibre style'), 'calibre')])
         r('book_list_tooltips', gprefs)
+        r('dnd_merge', gprefs)
+        r('wrap_toolbar_text', gprefs, restart_required=True)
         r('show_layout_buttons', gprefs, restart_required=True)
         r('row_numbers_in_book_list', gprefs)
         r('tag_browser_old_look', gprefs)
@@ -426,6 +427,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('emblem_position', gprefs, choices=[
             (_('Left'), 'left'), (_('Top'), 'top'), (_('Right'), 'right'), (_('Bottom'), 'bottom')])
         r('book_list_extra_row_spacing', gprefs)
+        r('booklist_grid', gprefs)
         r('book_details_narrow_comments_layout', gprefs, choices=[(_('Float'), 'float'), (_('Columns'), 'columns')])
         self.opt_book_details_narrow_comments_layout.setToolTip(textwrap.fill(_(
             'Choose how the text is laid out when using the "Narrow" user interface layout.'
@@ -478,11 +480,11 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         r('tag_browser_dont_collapse', gprefs, setting=CommaSeparatedList)
 
         choices = set([k for k in db.field_metadata.all_field_keys()
-                if (db.field_metadata[k]['is_category'] and
-                   (db.field_metadata[k]['datatype'] in ['text', 'series', 'enumeration']) and
-                    not db.field_metadata[k]['display'].get('is_names', False)) or
-                   (db.field_metadata[k]['datatype'] in ['composite'] and
-                    db.field_metadata[k]['display'].get('make_category', False))])
+                if (db.field_metadata[k]['is_category'] and (
+                    db.field_metadata[k]['datatype'] in ['text', 'series', 'enumeration'
+                    ]) and not db.field_metadata[k]['display'].get('is_names', False)) or (
+                    db.field_metadata[k]['datatype'] in ['composite'
+                    ] and db.field_metadata[k]['display'].get('make_category', False))])
         choices -= set(['authors', 'publisher', 'formats', 'news', 'identifiers'])
         choices |= set(['search'])
         self.opt_categories_using_hierarchy.update_items_cache(choices)
@@ -501,19 +503,19 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
                 self.field_display_order)
         self.display_model.dataChanged.connect(self.changed_signal)
         self.field_display_order.setModel(self.display_model)
-        self.df_up_button.clicked.connect(partial(move_field_up,
-                                  self.field_display_order, self.display_model))
-        self.df_down_button.clicked.connect(partial(move_field_down,
-                                  self.field_display_order, self.display_model))
+        connect_lambda(self.df_up_button.clicked, self,
+                lambda self: move_field_up(self.field_display_order, self.display_model))
+        connect_lambda(self.df_down_button.clicked, self,
+                lambda self: move_field_down(self.field_display_order, self.display_model))
 
         self.qv_display_model = QVDisplayedFields(self.gui.current_db,
                 self.qv_display_order)
         self.qv_display_model.dataChanged.connect(self.changed_signal)
         self.qv_display_order.setModel(self.qv_display_model)
-        self.qv_up_button.clicked.connect(partial(move_field_up,
-                                  self.qv_display_order, self.qv_display_model))
-        self.qv_down_button.clicked.connect(partial(move_field_down,
-                                  self.qv_display_order, self.qv_display_model))
+        connect_lambda(self.qv_up_button.clicked, self,
+                lambda self: move_field_up(self.qv_display_order, self.qv_display_model))
+        connect_lambda(self.qv_down_button.clicked, self,
+                lambda self: move_field_down(self.qv_display_order, self.qv_display_model))
 
         self.edit_rules = EditRules(self.tabWidget)
         self.edit_rules.changed.connect(self.changed_signal)
@@ -556,8 +558,8 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         b.clicked.connect(self.restore_cover_grid_appearance)
         self.cover_grid_empty_cache.clicked.connect(self.empty_cache)
         self.cover_grid_open_cache.clicked.connect(self.open_cg_cache)
-        self.cover_grid_smaller_cover.clicked.connect(partial(self.resize_cover, True))
-        self.cover_grid_larger_cover.clicked.connect(partial(self.resize_cover, False))
+        connect_lambda(self.cover_grid_smaller_cover.clicked, self, lambda self: self.resize_cover(True))
+        connect_lambda(self.cover_grid_larger_cover.clicked, self, lambda self: self.resize_cover(False))
         self.cover_grid_reset_size.clicked.connect(self.cg_reset_size)
         self.opt_cover_grid_disk_cache_size.setMinimum(self.gui.grid_view.thumbnail_cache.min_disk_cache)
         self.opt_cover_grid_disk_cache_size.setMaximum(self.gui.grid_view.thumbnail_cache.min_disk_cache * 100)
@@ -722,8 +724,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         name = unicode(fi.family())
 
         self.font_display.setFont(font)
-        self.font_display.setText(name +
-                ' [%dpt]'%fi.pointSize())
+        self.font_display.setText(name + ' [%dpt]'%fi.pointSize())
 
     def change_font(self, *args):
         fd = QFontDialog(self.build_font_obj(), self)
@@ -765,6 +766,7 @@ class ConfigWidget(ConfigWidgetBase, Ui_Form):
         gui.tags_view.set_look_and_feel()
         gui.tags_view.reread_collapse_parameters()
         gui.library_view.refresh_book_details()
+        gui.library_view.refresh_grid()
         gui.library_view.set_row_header_visibility()
         gui.cover_flow.setShowReflections(gprefs['cover_browser_reflections'])
         gui.cover_flow.setPreserveAspectRatio(gprefs['cb_preserve_aspect_ratio'])
