@@ -8,13 +8,16 @@ import os
 import textwrap
 import time
 
-import sip
 from PyQt5.Qt import (
     QCheckBox, QComboBox, QDialog, QDialogButtonBox, QDoubleSpinBox, QFormLayout,
     QFrame, QHBoxLayout, QIcon, QLabel, QLineEdit, QListWidget, QPlainTextEdit,
     QPushButton, QScrollArea, QSize, QSizePolicy, QSpinBox, Qt, QTabWidget, QTimer,
-    QUrl, QVBoxLayout, QWidget, pyqtSignal
+    QToolButton, QUrl, QVBoxLayout, QWidget, pyqtSignal
 )
+try:
+    from PyQt5 import sip
+except ImportError:
+    import sip
 
 from calibre import as_unicode
 from calibre.gui2 import (
@@ -31,7 +34,6 @@ from calibre.srv.users import (
 )
 from calibre.utils.icu import primary_sort_key
 
-
 # Advanced {{{
 
 
@@ -104,6 +106,7 @@ class Text(QLineEdit):
 
     def __init__(self, name, layout):
         QLineEdit.__init__(self)
+        self.setClearButtonEnabled(True)
         opt = options[name]
         self.textChanged.connect(self.changed_signal.emit)
         init_opt(self, opt, layout)
@@ -115,6 +118,40 @@ class Text(QLineEdit):
         self.setText(type(u'')(val or ''))
 
 
+class Path(QWidget):
+
+    changed_signal = pyqtSignal()
+
+    def __init__(self, name, layout):
+        QWidget.__init__(self)
+        self.dname = name
+        opt = options[name]
+        self.l = l = QHBoxLayout(self)
+        l.setContentsMargins(0, 0, 0, 0)
+        self.text = t = QLineEdit(self)
+        t.setClearButtonEnabled(True)
+        t.textChanged.connect(self.changed_signal.emit)
+        l.addWidget(t)
+
+        self.b = b = QToolButton(self)
+        l.addWidget(b)
+        b.setIcon(QIcon(I('document_open.png')))
+        b.setToolTip(_("Browse for the file"))
+        b.clicked.connect(self.choose)
+        init_opt(self, opt, layout)
+
+    def get(self):
+        return self.text.text().strip() or None
+
+    def set(self, val):
+        self.text.setText(type(u'')(val or ''))
+
+    def choose(self):
+        ans = choose_files(self, 'choose_path_srv_opts_' + self.dname, _('Choose a file'), select_only_single_file=True)
+        if ans:
+            self.set(ans[0])
+
+
 class Choices(QComboBox):
 
     changed_signal = pyqtSignal()
@@ -147,6 +184,7 @@ class AdvancedTab(QWidget):
         self.l = l = QFormLayout(self)
         l.setFieldGrowthPolicy(l.AllNonFixedFieldsGrow)
         self.widgets = []
+        self.widget_map = {}
         self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         for name in sorted(options, key=lambda n: options[n].shortdoc.lower()):
             if name in ('auth', 'port', 'allow_socket_preallocation', 'userdb'):
@@ -162,9 +200,12 @@ class AdvancedTab(QWidget):
                 w = Float
             else:
                 w = Text
+                if name in ('ssl_certfile', 'ssl_keyfile'):
+                    w = Path
             w = w(name, l)
             setattr(self, 'opt_' + name, w)
             self.widgets.append(w)
+            self.widget_map[name] = w
 
     def genesis(self):
         opts = server_config()
@@ -176,10 +217,16 @@ class AdvancedTab(QWidget):
         for w in self.widgets:
             w.set(w.default_val)
 
+    def get(self, name):
+        return self.widget_map[name].get()
+
     @property
     def settings(self):
         return {w.name: w.get() for w in self.widgets}
 
+    @property
+    def has_ssl(self):
+        return bool(self.get('ssl_certfile')) and bool(self.get('ssl_keyfile'))
 
 # }}}
 
@@ -632,7 +679,7 @@ class User(QWidget):
             b = _('Change the blocked libraries')
         else:
             m = _('{} is currently allowed access to all libraries')
-            b = _('Restrict the &libraries {} can access'.format(self.username))
+            b = _('Restrict the &libraries {} can access').format(self.username)
         self.restrict_button.setText(b),
         self.access_label.setText(m.format(username))
 
@@ -975,10 +1022,14 @@ class ConfigWidget(ConfigWidgetBase):
         self.stopping_msg.accept()
 
     def test_server(self):
-        prefix = self.advanced_tab.opt_url_prefix.text().strip()
-        open_url(
-            QUrl('http://127.0.0.1:' + str(self.main_tab.opt_port.value()) + prefix)
-        )
+        prefix = self.advanced_tab.get('url_prefix') or ''
+        protocol = 'https' if self.advanced_tab.has_ssl else 'http'
+        lo = self.advanced_tab.get('listen_on') or '0.0.0.0'
+        lo = {'0.0.0.0': '127.0.0.1', '::':'::1'}.get(lo)
+        url = '{protocol}://{interface}:{port}{prefix}'.format(
+            protocol=protocol, interface=lo,
+            port=self.main_tab.opt_port.value(), prefix=prefix)
+        open_url(QUrl(url))
 
     def view_server_logs(self):
         from calibre.srv.embedded import log_paths
@@ -1011,6 +1062,25 @@ class ConfigWidget(ConfigWidgetBase):
         bx = QDialogButtonBox(QDialogButtonBox.Ok)
         layout.addWidget(bx)
         bx.accepted.connect(d.accept)
+        b = bx.addButton(_('&Clear logs'), bx.ActionRole)
+
+        def clear_logs():
+            if getattr(self.server, 'is_running', False):
+                return error_dialog(d, _('Server running'), _(
+                    'Cannot clear logs while the server is running. First stop the server.'), show=True)
+            if self.server:
+                self.server.access_log.clear()
+                self.server.log.clear()
+            else:
+                for x in (log_error_file, log_access_file):
+                    try:
+                        os.remove(x)
+                    except EnvironmentError as err:
+                        if err.errno != errno.ENOENT:
+                            raise
+            el.setPlainText(''), al.setPlainText('')
+
+        b.clicked.connect(clear_logs)
         d.show()
 
     def save_changes(self):
