@@ -239,14 +239,14 @@ class Diff(Dialog):
         r = l.rowCount()
         self.bp = b = QToolButton(self)
         b.setIcon(QIcon(I('back.png')))
-        b.clicked.connect(partial(self.view.next_change, -1))
+        connect_lambda(b.clicked, self, lambda self: self.view.next_change(-1))
         b.setToolTip(_('Go to previous change') + ' [p]')
         b.setText(_('&Previous change')), b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
         l.addWidget(b, r, 0)
 
         self.bn = b = QToolButton(self)
         b.setIcon(QIcon(I('forward.png')))
-        b.clicked.connect(partial(self.view.next_change, 1))
+        connect_lambda(b.clicked, self, lambda self: self.view.next_change(1))
         b.setToolTip(_('Go to next change') + ' [n]')
         b.setText(_('&Next change')), b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
         l.addWidget(b, r, 1)
@@ -255,16 +255,16 @@ class Diff(Dialog):
         s.initialize('diff_search_history')
         l.addWidget(s, r, 2)
         s.setPlaceholderText(_('Search for text'))
-        s.returnPressed.connect(partial(self.do_search, False))
+        connect_lambda(s.returnPressed, self, lambda self: self.do_search(False))
         self.sbn = b = QToolButton(self)
         b.setIcon(QIcon(I('arrow-down.png')))
-        b.clicked.connect(partial(self.do_search, False))
+        connect_lambda(b.clicked, self, lambda self: self.do_search(False))
         b.setToolTip(_('Find next match'))
         b.setText(_('Next &match')), b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
         l.addWidget(b, r, 3)
         self.sbp = b = QToolButton(self)
         b.setIcon(QIcon(I('arrow-up.png')))
-        b.clicked.connect(partial(self.do_search, True))
+        connect_lambda(b.clicked, self, lambda self: self.do_search(True))
         b.setToolTip(_('Find previous match'))
         b.setText(_('P&revious match')), b.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
         l.addWidget(b, r, 4)
