@@ -27,7 +27,7 @@ from calibre.gui2.tweak_book.widgets import Dialog
 from calibre.gui2.tweak_book.file_list import name_is_ok
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.utils.localization import get_lang, canonicalize_lang
-from calibre.utils.icu import sort_key
+from calibre.utils.icu import numeric_sort_key
 
 
 class ChooseName(Dialog):  # {{{
@@ -155,7 +155,7 @@ class Images(QAbstractListModel):
         self.image_names = []
         self.image_cache = {}
         if c is not None:
-            for name in sorted(c.mime_map, key=sort_key):
+            for name in sorted(c.mime_map, key=numeric_sort_key):
                 if c.mime_map[name].startswith('image/'):
                     self.image_names.append(name)
 
@@ -259,14 +259,21 @@ class InsertImage(Dialog):
             self.preserve_aspect_ratio = a = QCheckBox(_('Preserve aspect ratio'))
             a.setToolTip(_('Preserve the aspect ratio of the inserted image when rendering it full paged'))
             a.setChecked(tprefs['preserve_aspect_ratio_when_inserting_image'])
-            f.toggled.connect(lambda : (tprefs.set('insert_full_screen_image', f.isChecked()), a.setVisible(f.isChecked())))
-            a.toggled.connect(lambda : tprefs.set('preserve_aspect_ratio_when_inserting_image', a.isChecked()))
+            f.toggled.connect(self.full_page_image_toggled)
+            a.toggled.connect(self.par_toggled)
             a.setVisible(f.isChecked())
             h = QHBoxLayout()
             l.addLayout(h, 3, 0, 1, -1)
             h.addWidget(f), h.addStretch(10), h.addWidget(a)
         l.addWidget(self.bb, 4, 0, 1, 2)
 
+    def full_page_image_toggled(self):
+        tprefs.set('insert_full_screen_image', self.fullpage.isChecked())
+        self.preserve_aspect_ratio.setVisible(self.fullpage.isChecked())
+
+    def par_toggled(self):
+        tprefs.set('preserve_aspect_ratio_when_inserting_image', self.preserve_aspect_ratio.isChecked())
+
     def refresh(self):
         self.d.cover_cache.clear()
         self.model.refresh()
@@ -369,7 +376,7 @@ class ChooseFolder(Dialog):  # {{{
 
         def process(node, parent):
             parent.setIcon(0, QIcon(I('mimetypes/dir.png')))
-            for child in sorted(node, key=sort_key):
+            for child in sorted(node, key=numeric_sort_key):
                 c = QTreeWidgetItem(parent, (child,))
                 process(node[child], c)
         process(create_folder_tree(current_container()), self.root)
@@ -439,9 +446,9 @@ class NewBook(Dialog):  # {{{
         bb.clear()
         bb.addButton(bb.Cancel)
         b = bb.addButton('&EPUB', bb.AcceptRole)
-        b.clicked.connect(partial(self.set_fmt, 'epub'))
+        connect_lambda(b.clicked, self, lambda self: self.set_fmt('epub'))
         b = bb.addButton('&AZW3', bb.AcceptRole)
-        b.clicked.connect(partial(self.set_fmt, 'azw3'))
+        connect_lambda(b.clicked, self, lambda self: self.set_fmt('azw3'))
 
     def set_fmt(self, fmt):
         self.fmt = fmt
